/*
 * The Relaxer artifact
 * Copyright (c) 2000-2004, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package jp.sourceforge.talisman.xmlcli.builder.relaxer;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>CommandLine</b> is generated from xmlcli-2_0.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="command-line">
 *       <attribute name="name">
 *         <data type="string"/>
 *       </attribute>
 * 
 *       <element name="definitions">
 *         <optional>
 *           <ref name="arguments"/>
 *         </optional>
 *         <optional>
 *           <ref name="options"/>
 *         </optional>
 *       </element>
 *       <element name="program-arguments">
 *         <ref name="arguments"/>
 *       </element>
 *       <element name="available-options">
 *         <zeroOrMore>
 *           <ref name="available-option"/>
 *         </zeroOrMore>
 *       </element>
 *     </element>-->
 * <!-- for javadoc -->
 * <pre> &lt;element name="command-line"&gt;
 *       &lt;attribute name="name"&gt;
 *         &lt;data type="string"/&gt;
 *       &lt;/attribute&gt;
 * 
 *       &lt;element name="definitions"&gt;
 *         &lt;optional&gt;
 *           &lt;ref name="arguments"/&gt;
 *         &lt;/optional&gt;
 *         &lt;optional&gt;
 *           &lt;ref name="options"/&gt;
 *         &lt;/optional&gt;
 *       &lt;/element&gt;
 *       &lt;element name="program-arguments"&gt;
 *         &lt;ref name="arguments"/&gt;
 *       &lt;/element&gt;
 *       &lt;element name="available-options"&gt;
 *         &lt;zeroOrMore&gt;
 *           &lt;ref name="available-option"/&gt;
 *         &lt;/zeroOrMore&gt;
 *       &lt;/element&gt;
 *     &lt;/element&gt;</pre>
 *
 * @version xmlcli-2_0.rng (Thu Aug 16 18:37:44 JST 2007)
 * @author  Relaxer 1.1b (http://www.relaxer.org)
 */
@SuppressWarnings({ "unchecked", "serial", "unused" })
public class CommandLine implements java.io.Serializable, Cloneable {
    private String name_;
    private CommandLineDefinitions commandLineDefinitions_;
    private CommandLineProgramArguments commandLineProgramArguments_;
    private CommandLineAvailableOptions commandLineAvailableOptions_;

    /**
     * Creates a <code>CommandLine</code>.
     *
     */
    public CommandLine() {
        name_ = "";
    }

    /**
     * Creates a <code>CommandLine</code>.
     *
     * @param source
     */
    public CommandLine(CommandLine source) {
        setup(source);
    }

    /**
     * Creates a <code>CommandLine</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public CommandLine(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>CommandLine</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public CommandLine(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>CommandLine</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public CommandLine(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>CommandLine</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>CommandLine</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>CommandLine</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>CommandLine</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>CommandLine</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>CommandLine</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public CommandLine(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>CommandLine</code> by the CommandLine <code>source</code>.
     *
     * @param source
     */
    public void setup(CommandLine source) {
        int size;
        name_ = source.name_;
        if (source.commandLineDefinitions_ != null) {
            setCommandLineDefinitions((CommandLineDefinitions)source.getCommandLineDefinitions().clone());
        }
        if (source.commandLineProgramArguments_ != null) {
            setCommandLineProgramArguments((CommandLineProgramArguments)source.getCommandLineProgramArguments().clone());
        }
        if (source.commandLineAvailableOptions_ != null) {
            setCommandLineAvailableOptions((CommandLineAvailableOptions)source.getCommandLineAvailableOptions().clone());
        }
    }

    /**
     * Initializes the <code>CommandLine</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>CommandLine</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>CommandLine</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        name_ = URelaxer.getAttributePropertyAsString(element, "name");
        setCommandLineDefinitions(new CommandLineDefinitions(stack));
        setCommandLineProgramArguments(new CommandLineProgramArguments(stack));
        setCommandLineAvailableOptions(new CommandLineAvailableOptions(stack));
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new CommandLine((CommandLine)this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("command-line");
        int size;
        if (this.name_ != null) {
            URelaxer.setAttributePropertyByString(element, "name", this.name_);
        }
        this.commandLineDefinitions_.makeElement(element);
        this.commandLineProgramArguments_.makeElement(element);
        this.commandLineAvailableOptions_.makeElement(element);
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>CommandLine</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>CommandLine</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>CommandLine</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>CommandLine</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>CommandLine</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>CommandLine</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>name</b>.
     *
     * @return String
     */
    public String getName() {
        return (name_);
    }

    /**
     * Sets the String property <b>name</b>.
     *
     * @param name
     */
    public void setName(String name) {
        this.name_ = name;
    }

    /**
     * Gets the CommandLineDefinitions property <b>commandLineDefinitions</b>.
     *
     * @return CommandLineDefinitions
     */
    public CommandLineDefinitions getCommandLineDefinitions() {
        return (commandLineDefinitions_);
    }

    /**
     * Sets the CommandLineDefinitions property <b>commandLineDefinitions</b>.
     *
     * @param commandLineDefinitions
     */
    public void setCommandLineDefinitions(CommandLineDefinitions commandLineDefinitions) {
        this.commandLineDefinitions_ = commandLineDefinitions;
    }

    /**
     * Gets the CommandLineProgramArguments property <b>commandLineProgramArguments</b>.
     *
     * @return CommandLineProgramArguments
     */
    public CommandLineProgramArguments getCommandLineProgramArguments() {
        return (commandLineProgramArguments_);
    }

    /**
     * Sets the CommandLineProgramArguments property <b>commandLineProgramArguments</b>.
     *
     * @param commandLineProgramArguments
     */
    public void setCommandLineProgramArguments(CommandLineProgramArguments commandLineProgramArguments) {
        this.commandLineProgramArguments_ = commandLineProgramArguments;
    }

    /**
     * Gets the CommandLineAvailableOptions property <b>commandLineAvailableOptions</b>.
     *
     * @return CommandLineAvailableOptions
     */
    public CommandLineAvailableOptions getCommandLineAvailableOptions() {
        return (commandLineAvailableOptions_);
    }

    /**
     * Sets the CommandLineAvailableOptions property <b>commandLineAvailableOptions</b>.
     *
     * @param commandLineAvailableOptions
     */
    public void setCommandLineAvailableOptions(CommandLineAvailableOptions commandLineAvailableOptions) {
        this.commandLineAvailableOptions_ = commandLineAvailableOptions;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<command-line");
        if (name_ != null) {
            buffer.append(" name=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.append("\"");
        }
        buffer.append(">");
        commandLineDefinitions_.makeTextElement(buffer);
        commandLineProgramArguments_.makeTextElement(buffer);
        commandLineAvailableOptions_.makeTextElement(buffer);
        buffer.append("</command-line>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<command-line");
        if (name_ != null) {
            buffer.write(" name=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.write("\"");
        }
        buffer.write(">");
        commandLineDefinitions_.makeTextElement(buffer);
        commandLineProgramArguments_.makeTextElement(buffer);
        commandLineAvailableOptions_.makeTextElement(buffer);
        buffer.write("</command-line>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<command-line");
        if (name_ != null) {
            buffer.print(" name=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.print("\"");
        }
        buffer.print(">");
        commandLineDefinitions_.makeTextElement(buffer);
        commandLineProgramArguments_.makeTextElement(buffer);
        commandLineAvailableOptions_.makeTextElement(buffer);
        buffer.print("</command-line>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getNameAsString() {
        return (URelaxer.getString(getName()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setNameByString(String string) {
        setName(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Gets attribute.
     *
     * @return String[]
     */
    public String[] getCommandLineAvailableOptions_AvailableOption() {
        if (commandLineAvailableOptions_ == null) {
            return (null);
        }
        return (commandLineAvailableOptions_.getAvailableOption());
    }

    /**
     * Gets attribute.
     *
     * @param availableOption
     */
    public void setCommandLineAvailableOptions_AvailableOption(String[] availableOption) {
        if (commandLineAvailableOptions_ == null) {
            commandLineAvailableOptions_ = new CommandLineAvailableOptions();
        }
        commandLineAvailableOptions_.setAvailableOption(availableOption);
    }

    /**
     * Adds attribute.
     *
     * @param availableOption
     */
    public void addCommandLineAvailableOptions_AvailableOption(String availableOption) {
        if (commandLineAvailableOptions_ == null) {
            commandLineAvailableOptions_ = new CommandLineAvailableOptions();
        }
        commandLineAvailableOptions_.addAvailableOption(availableOption);
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>CommandLine</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "command-line")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "name")) {
            return (false);
        }
        $match$ = true;
        if (!CommandLineDefinitions.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!CommandLineProgramArguments.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!CommandLineAvailableOptions.isMatchHungry(target)) {
            return (false);
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>CommandLine</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>CommandLine</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
